/*
//to play Button at any time... 
//cahnge the library "Adafruit_PN532.h"
//Windows: ... C:\Users\scott\Documents\Arduino\libraries\Adafruit_PN532 (Line 172, timeot= min. 50), recommended timeout = 100
//card readable any time
//with for loop just readable if time run out to play

This sketch demonstrates recording of standard WAV files that can be played on any device that supports WAVs. The recording
uses a single ended input from any of the analog input pins. Uses AVCC (5V) reference currently.

Requirements:
Class 4 or 6 SD Card
Audio Input Device (Microphone, etc)
Arduino Uno,Nano, Mega, etc.

Steps:
1. Edit pcmConfig.h
    a: On Uno or non-mega boards, #define buffSize 128. May need to increase.
    b: Uncomment #define ENABLE_RECORDING and #define BLOCK_COUNT 10000UL

2. Usage is as below. See https://github.com/TMRh20/TMRpcm/wiki/Advanced-Features#wiki-recording-audio for
   additional informaiton.

Notes: Recording will not work in Multi Mode.
Performance is very dependant on SD write speed, and memory used.
Better performance may be seen using the SdFat library. See included example for usage.
Running the Arduino from a battery or filtered power supply will reduce noise.
*/

//#include "Arduino.h"

#include <SD.h>
#include <SPI.h>
#include <TMRpcm.h>

//--------NFC
#include <Wire.h>
#include <Adafruit_PN532.h>

//I2C Connection
#define PN532_IRQ   (2)
#define PN532_RESET (3)  // Not connected by default on the NFC Shield

Adafruit_PN532 nfc(PN532_IRQ, PN532_RESET);

//int relevantPage = 5;
//int pageNumberToRead = 10; //read all pages or just a few??


// --------SD Card audio
#define SD_ChipSelectPin 10  //using digital pin 4 on arduino nano 328, can use other pins

TMRpcm audio;   // create an object for use in this sketch 
int recordCounter = 0;
int SDfileCounter = 0;

const char *first = "rec";
const char *zero = "0";
const char *last  = ".wav";

char lastTrack[16];

// --------BUTTON 
#define recordButton  6
int recordButtonState = 1;
int lastRecordButtonState = 1;
int lastPlayButtonState = 1;

#define playButton  5
int playButtonState = 1;

// --------VIBRATION 
#define vibration  8

        
//##########################################################################################SETUP
void setup() {

  // --------BUTTON 
  pinMode(recordButton,INPUT_PULLUP);
  pinMode(playButton,INPUT_PULLUP);

  // --------VIBRATION 
  pinMode(vibration,OUTPUT);// define a pin as output

  // --------START SERIAL
  Serial.begin(115200);
  
  //---------COUNT SD FILES
  //get num of files on sd card to use as record counter
  //recordIdFileCount = getFileCount("/");
  pinMode(10, OUTPUT);
  SD.begin(10);
  SDfileCounter = getFileCount( "/" );
 
  // --------SD Card audio
  audio.speakerPin = 9; //5,6,11 or 46 on Mega, 9 on Uno, Nano, etc
  audio.setVolume(4);
  
  if (!SD.begin(SD_ChipSelectPin)) {
    Serial.println(F("Mistake occured, SD card inserted?"));   
    return;
  }else{
    Serial.println(F("SD OK")); 
  }
  // The audio library needs to know which CS pin to use for recording
  audio.CSPin = SD_ChipSelectPin;

  //--------NFC
  nfc.begin();

  uint32_t versiondata = nfc.getFirmwareVersion();
  if (! versiondata) {
    Serial.print(F("Didn't find PN53x board"));
    while (1); // halt
  }
  // Got ok data, print it out!
  //Serial.print("Found chip PN5"); Serial.println((versiondata>>24) & 0xFF, HEX); 
  //Serial.print("Firmware ver. "); Serial.print((versiondata>>16) & 0xFF, DEC); 
  //Serial.print('.'); Serial.println((versiondata>>8) & 0xFF, DEC);
  
  // configure board to read RFID tags
  nfc.SAMConfig();
  
  Serial.println(F("Waiting for an ISO14443A Card ..."));
  
}

//##########################################################################################LOOP
void loop() {

    // --------BUTTONS 
    recordButtonState = digitalRead(recordButton);
    playButtonState = digitalRead(playButton); 
 
    //Serial.println(recordButtonState);

    if(recordButtonState != lastRecordButtonState){

      if(recordButtonState == false && playButtonState != false){
        Serial.println(F("recording..."));
        startRecAndSave();
      }
      if(recordButtonState != false && playButtonState != false && lastPlayButtonState != lastRecordButtonState){
        Serial.print(F("recording stopped: "));
        Serial.println(lastTrack);
        audio.stopRecording(lastTrack);
      }
   }
   lastRecordButtonState = recordButtonState;

    //play LAST audio if Button is pressed
    if(playButtonState == false && recordButtonState != false && lastPlayButtonState != lastRecordButtonState){
      Serial.print(F("play audio: "));
      Serial.println(lastTrack);
      audio.play(lastTrack);
    }
    lastPlayButtonState = playButtonState;
    
   //--------------------------------------------------------------------------------------------NFC
    uint8_t success;
    uint8_t uid[] = { 0, 0, 0, 0, 0, 0, 0 };  // Buffer to store the returned UID
    uint8_t uidLength;                        // Length of the UID (4 or 7 bytes depending on ISO14443A card type)
    
    success = nfc.readPassiveTargetID(PN532_MIFARE_ISO14443A, uid, &uidLength);
    
    if (success) {
      
      // Display some basic information about the card
      Serial.println(F("Found an ISO14443A card"));
      Serial.print(F("  UID Length: "));Serial.print(uidLength, DEC);Serial.println(" bytes");
      Serial.print(F("  UID Value: "));
      nfc.PrintHex(uid, uidLength);
      Serial.println("");

    //----------------------------------------------------------------MIFARE CARDS
    if (uidLength == 4){
      
      Serial.println(F("Seems to be a Mifare Classic card (4 byte UID)")); 
      Serial.println(F("Trying to authenticate block 4 with default KEYA value"));
      uint8_t keya[6] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
      success = nfc.mifareclassic_AuthenticateBlock(uid, uidLength, 4, 0, keya);

       if (success){
        
        Serial.println(F("Sector 1 (Blocks 4..7) has been authenticated"));
        uint8_t data[16];

        // --------ASSIGN TO NFC CHIP--------------
        //if both Buttons are pressed
        if(playButtonState == false && recordButtonState == false){
      
           Serial.println(F("both buttons pressed")); 

            Serial.print(F("lastTrack var printed (write uid 4 function): "));
            Serial.println(lastTrack);
            
            // Write a block of data to the card. Block number should be the block to write and data
            // should be a byte array of length 16 with the data to write. If the data is successfully
            // written then True is returned, otherwise False is returned.
            //data[16]= { 'a', 'd', 'a', 'f', 'r', 'u', 'i', 't', '.', 'c', 'o', 'm', 0, 0, 0, 0 };
            
            //write last recording  
            //data[16]= { 'r', 'e', 'c', '0', '0', '0', '0', '0', '0', '.', 'w', 'a', 'v', 0 , 0, 0 };
            uint8_t data[16]= { lastTrack[0], lastTrack[1], lastTrack[2],lastTrack[3], lastTrack[4],
                                lastTrack[5], lastTrack[6], lastTrack[7], lastTrack[8], lastTrack[9],
                                lastTrack[10], lastTrack[11], lastTrack[12], 0,
                                0, 0};
      
            //try to write NFC Tag
            uint8_t success = nfc.mifareclassic_WriteDataBlock(4, data);

            Serial.println(F("Wait 1000 before writing the card again"));
            delay(1000);
            
            if(success){
              Serial.println(F("Data written to Mifare Tag"));
            }else{
              Serial.println(F("Card data not valid"));
            }    
          }
        
        //------------------------------------------

        success = nfc.mifareclassic_ReadDataBlock(4, data);

        if (success){
          
          // Data seems to have been read ... spit it out
          Serial.println(F("Reading Block 4:"));
          nfc.PrintHexChar(data, 16);
          Serial.println(F(""));

          //assign last Track to saved track on NFC chip
          for(uint8_t i=0; i<16 ; i++){
            lastTrack[i] = data[i];
          }

          Serial.print(F("Saved track from NFC to play (read nfc function uid 4): "));
          Serial.println(lastTrack);

          //vibration
          digitalWrite(vibration,HIGH);
          delay(600);
          digitalWrite(vibration, LOW);
          delay(600); 
          
          Serial.println(F("Wait 1000 before reading the card again"));
          delay(1000);
          
        }else{
          Serial.println(F("Ooops ... unable to read the requested block.  Try another key?"));
        }
      }else{
        Serial.println(F("Ooops ... authentication failed: Try another key?"));
      }
    }

//----------------------------------------------------------------NTAG2xx CARDS

    if (uidLength == 7){
      
    //uint8_t data[32];
    uint8_t data1[4];
    uint8_t data2[4];
    uint8_t data3[4];

     uint8_t  success1;
     uint8_t  success2;
     uint8_t  success3;
      
      //bool writeOnce = false;
      //bool readOnce = false;
 
      Serial.println(F("Seems to be an NTAG2xx tag (7 byte UID)"));  

      //write chip if both buttons are pressed
       if(playButtonState == false && recordButtonState == false){
          
          Serial.println(F("Both buttons pressed."));

          Serial.print(F("lastTrack var printed (write uid 7 function): "));
          Serial.println(lastTrack);


          uint8_t data1[4] = {lastTrack[0], lastTrack[1], lastTrack[2],lastTrack[3]};
          
          uint8_t data2[4] = {lastTrack[4],lastTrack[5],lastTrack[6], lastTrack[7]};

          uint8_t data3[4] = {lastTrack[8], lastTrack[9],lastTrack[10], lastTrack[11]};

            //try to write NFC Tag
            success1 = nfc.ntag2xx_WritePage(5, data1);
            success2 = nfc.ntag2xx_WritePage(6, data2);
            success3 = nfc.ntag2xx_WritePage(7, data3);
  
            //writeOnce = true; //write just once assured

            Serial.println(F("Wait 1000 before writing the card again"));
            delay(1000);

          if(!success1 && !success2 && !success3){
            Serial.println(F("Card data not valid"));
          } 
        }

        success1 = nfc.ntag2xx_ReadPage(5, data1);
        success2 = nfc.ntag2xx_ReadPage(6, data2);
        success3 = nfc.ntag2xx_ReadPage(7, data3);

        if (success1 && success2 && success3){

           Serial.println(F("Read Ntag2xx: "));
           
          // Dump the page data
          Serial.print(F("5 PAGE: "));
          nfc.PrintHexChar(data1, 4);
          Serial.print(F("6 PAGE: "));
          nfc.PrintHexChar(data2, 4);
          Serial.print(F("7 PAGE: "));
          nfc.PrintHexChar(data3, 4);

          for(uint8_t i=0; i<4 ; i++){
            lastTrack[i] = data1[i];
            lastTrack[i+4] = data2[i];
            lastTrack[i+8] = data3[i];
          }

          //vibration
          digitalWrite(vibration,HIGH);
          delay(600);
          digitalWrite(vibration, LOW);
          delay(600); 
          
          Serial.print(F("lastTrack after read..."));
          Serial.println(lastTrack);
          
        }
        
      } //uid length 7 end
    } //if success all
    else
    {
      //Serial.println(F("This doesn't seem to be an NTAG203 tag (UUID length != 7 bytes)!"));
    } //if success authenticate card
 
} //void loop end


  
//##########################################################################################FUNCTIONS
void startRecAndSave(){

    //restart counting after new start of device...
    //use number of existing files to count, maybe special dir for recordings?

    char myConcatenation[16];
    int recCount = SDfileCounter + recordCounter;
    char num[16];
    itoa(recCount, num, 10); //itoa(value,str,base)

    int zeroPart =  0;
    if(recCount < 10 && recCount >= 0){
       zeroPart = 4;
    }
    if(recCount < 100 && recCount >= 10){
       zeroPart = 3;
    }
    if(recCount < 1000 && recCount >= 1000){
       zeroPart = 2;
    }
    if(recCount < 10000 && recCount >= 1000){
       zeroPart = 1;
    }
    if(recCount < 999999 && recCount >= 10000){
       zeroPart = 0;
    }
  
    //concatenate char Array to record name
    strcpy(myConcatenation,first);
    
    for(uint8_t i=0; i<zeroPart; i++){
      strcat(myConcatenation,zero);
    }
    strcat(myConcatenation,num);
  
    strcat(myConcatenation,last);

    //Serial.print(F("concatination: "));
    //Serial.println(myConcatenation);
    
    audio.startRecording(myConcatenation,16000,A0);
    
    //save last track into myConcatination
    strcpy(lastTrack, myConcatenation);
    
    recordCounter++;
}

//--------------------------------------------

int getFileCount( String dir ){
  
  File d = SD.open(dir);

  if(!d){
    Serial.println(F("no entries found on SD card"));
  }
  
  int count_files = 0;
  char *nowTrack;
  
  while( true )
  {
  
    File entry =  d.openNextFile();
    
    if( !entry ){
      break;
    }
    
    count_files++;

    //Serial.print(F("entry found (file count function): "));
    //Serial.println(entry.name());
    
    nowTrack = entry.name();

    Serial.print(F("nowTrack (file count function): "));
    Serial.println(nowTrack);
  
    if(nowTrack){
      strcpy(lastTrack, nowTrack);
    }

   Serial.print(F("last Track after strcpy (file count function): "));
   Serial.println(lastTrack);

    entry.close();
  }

  //Serial.print(F("lastTrack is now (file count function): "));
  //Serial.println(lastTrack);

  d.close();

  //Serial.print(F("Number of Files on SD card (file count function): "));
  //Serial.println(count_files);
  return count_files;
  
}
